﻿Imports System.Windows.Media.Media3D
Imports MindFusion.Charting.Wpf
Imports System.Windows.Threading

Class Window1

    Dim brush As RadialGradientBrush

    Public Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.

        brush = New RadialGradientBrush(Colors.White, Colors.SteelBlue)
        brush.GradientOrigin = New Point(0.3, 0.3)
        brush.RadiusX = 0.7
        brush.RadiusY = 0.7

        Dim c As Brush = Brushes.LightSteelBlue
        Dim b As Brush = Brushes.SteelBlue

        Dim series As SurfaceSeries = New SurfaceSeries()
        series.Fills.Add(b)
        series.Strokes.Clear()
        series.Strokes.Add(c)
        series.ToolTipType = ToolTipType.ChartData
        surfaceChart.Series.Add(series)

        surfaceChart.XAxisSettings.LabelFontSize = 11
        surfaceChart.XAxisSettings.LabelOffset = 3
        surfaceChart.XAxisSettings.LabelFormat = "0.##"
        surfaceChart.XAxisSettings.TitleFontSize = 11
        surfaceChart.XAxisSettings.TitleOffset = 5

        surfaceChart.YAxisSettings.LabelFontSize = 11
        surfaceChart.YAxisSettings.LabelOffset = 3
        surfaceChart.YAxisSettings.LabelFormat = "0.##"
        surfaceChart.YAxisSettings.TitleFontSize = 11
        surfaceChart.YAxisSettings.TitleOffset = 5
        surfaceChart.YAxisSettings.MinValue = -10
        surfaceChart.YAxisSettings.MaxValue = 10

        surfaceChart.ZAxisSettings.LabelFontSize = 11
        surfaceChart.ZAxisSettings.LabelOffset = 3
        surfaceChart.ZAxisSettings.LabelFormat = "0.##"
        surfaceChart.ZAxisSettings.TitleFontSize = 11
        surfaceChart.ZAxisSettings.TitleOffset = 5


        surfaceChart.SideWallThickness = 0.5

        Dim light As DirectionalLight = New DirectionalLight()
        light.Color = Colors.White
        light.Direction = New Vector3D(-1, 3, -1)
        surfaceChart.StaticLights.Add(Light)

        RandomData(True)
    End Sub

    Private Function AlignPoint3D(ByVal p As Point3D) As Point3D
        Dim d As Double = surfaceChart.PointMergeThreshold
        Dim x As Double = 30
        Dim z As Double = 30

        If p.X < d Then
            p.X = 0
        End If
        If p.X > x - d Then
            p.X = x
        End If
        If p.Z < d Then
            p.Z = 0
        End If
        If p.Z > z - d Then
            p.Z = z
        End If

        Return p
    End Function

    Private Sub RandomData(ByVal newElevations As Boolean)
        surfaceChart.BeginInit()
        surfaceChart.PointMergeThreshold = pointMergeThreshold.Value
        surfaceChart.InterpolationSensitivity = sensitivity.Value
        If shape.SelectedIndex = 0 Then
            surfaceChart.InterpolationType = InterpolationType.Bezier
        ElseIf shape.SelectedIndex = 1 Then
            surfaceChart.InterpolationType = InterpolationType.Linear
        Else
            surfaceChart.InterpolationType = InterpolationType.None
        End If
        Dim r As Random = New Random()
        points.Clear()
        For i As Integer = 0 To CType(_randomData.Value - 1, Integer)
            Dim x As Double = r.NextDouble() * 30
            Dim z As Double = r.NextDouble() * 30
            Dim p As Point3D = AlignPoint3D(New Point3D(x, 0, z))
            points.Add(p)
        Next
        points.Add(New Point3D(0, 0, 30))
        points.Add(New Point3D(30, 0, 30))
        points.Add(New Point3D(30, 0, 0))
        points.Add(New Point3D(0, 0, 0))
        If newElevations Then

            elevations.Clear()
            For i As Integer = 0 To 19
                elevations.Add(New Point3D(r.Next(0, 30), r.Next(-10, 10), r.Next(0, 30)))
            Next
        End If
        CreateElevations(points, elevations, surfaceChart.InterpolationSensitivity)
        For Each s As SurfaceSeries In surfaceChart.Series
            s.XData.Clear()
            s.YData.Clear()
            s.ZData.Clear()
            For i As Integer = 0 To points.Count - 1
                Dim p As Point3D = points(i)
                s.XData.Add(p.X)
                s.YData.Add(p.Y)
                s.ZData.Add(p.Z)
            Next
        Next
        surfaceChart.EndInit()
    End Sub

    Private Sub CreateElevations(ByVal points As List(Of Point3D), _
        ByVal elevations As List(Of Point3D), ByVal sensitivity As Double)
        Dim piCoef As Double = Math.PI * 3 / 2
        Dim rCoef As Double = 10 / Math.PI
        For j As Integer = 0 To elevations.Count - 1
            Dim dataX As Double = elevations(j).X
            Dim dataZ As Double = elevations(j).Z
            Dim dataY As Double = elevations(j).Y
            For i As Integer = 0 To points.Count - 1
                Dim p As Point3D = points(i)
                Dim dx As Double = dataX - p.X
                Dim dz As Double = dataZ - p.Z
                Dim distance As Double = Math.Sqrt(dx * dx + dz * dz)
                If distance <= sensitivity Then
                    Dim percent As Double = 1.0 - distance / sensitivity
                    Dim coef As Double = dataY - p.Y
                    Dim elevate As Double = coef * (Math.Sin(percent * rCoef + piCoef) + 1.0) / 2
                    p.Y += elevate
                    points(i) = p
                End If
            Next
        Next
    End Sub

    Private Sub button2_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        RandomData(True)
    End Sub

    Private Sub viewport3D_MouseUp(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
        surfaceChart.ReleaseMouseCapture()
    End Sub

    Private Sub CheckBoxChecked(ByVal sender As Object, ByVal e As RoutedEventArgs)
        If surfaceChart Is Nothing Then
            Return
        End If
        Dim box As CheckBox = CType(sender, CheckBox)
        If box.Name = "showSurface" Then
            surfaceChart.ShowSurface = True
        ElseIf box.Name = "showScatters" Then
            surfaceChart.ShowScatters = True
        ElseIf box.Name = "showWireFrame" Then
            surfaceChart.ShowWireFrame = True
        ElseIf box.Name = "showXAxis" Then
            surfaceChart.XAxisSettings.LabelType = LabelType.AutoScale
        ElseIf box.Name = "showYAxis" Then
            surfaceChart.YAxisSettings.LabelType = LabelType.AutoScale
        ElseIf box.Name = "showZAxis" Then
            surfaceChart.ZAxisSettings.LabelType = LabelType.AutoScale
        ElseIf box.Name = "closeModelFigure" Then
            surfaceChart.CloseModelFigure = True
        End If
    End Sub

    Private Sub CheckBoxUnchecked(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Dim box As CheckBox = CType(sender, CheckBox)
        If box.Name = "showSurface" Then
            surfaceChart.ShowSurface = False
        ElseIf box.Name = "showScatters" Then
            surfaceChart.ShowScatters = False
        ElseIf box.Name = "showWireFrame" Then
            surfaceChart.ShowWireFrame = False
        ElseIf box.Name = "showXAxis" Then
            surfaceChart.XAxisSettings.LabelType = LabelType.Empty
        ElseIf box.Name = "showYAxis" Then
            surfaceChart.YAxisSettings.LabelType = LabelType.Empty
        ElseIf box.Name = "showZAxis" Then
            surfaceChart.ZAxisSettings.LabelType = LabelType.Empty
        ElseIf box.Name = "closeModelFigure" Then
            surfaceChart.CloseModelFigure = False
        End If
    End Sub

    Private Sub ComboBoxSelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
        If surfaceChart Is Nothing Then
            Return
        End If
        Dim box As ComboBox = CType(sender, ComboBox)

        If box.Name = "cameraType" Then
            If box.SelectedIndex = 0 Then
                surfaceChart.CameraType = MindFusion.Charting.Wpf.CameraType.Perspective
            Else
                surfaceChart.CameraType = MindFusion.Charting.Wpf.CameraType.Orthographic
            End If
        ElseIf box.Name = "shape" Then
            If box.SelectedIndex = 0 Then
                surfaceChart.InterpolationType = InterpolationType.Bezier
            ElseIf box.SelectedIndex = 1 Then
                surfaceChart.InterpolationType = InterpolationType.Linear
            Else
                RandomData(False)
            End If
        ElseIf box.Name = "textureType" Then
            If box.SelectedIndex = 0 Then
                surfaceChart.TextureType = TextureType.Brush
            Else
                surfaceChart.TextureType = TextureType.GradientColorMap
            End If
        ElseIf box.Name = "effect3D" Then
            If box.SelectedIndex = 0 Then
                surfaceChart.Effect3D = ChartEffect3D.None
            Else
                surfaceChart.Effect3D = ChartEffect3D.ShaderEffect
            End If
        End If
    End Sub

    Private Sub flatSurface_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        CType(surfaceChart.Series(0), SurfaceSeries).XData.Clear()
        CType(surfaceChart.Series(0), SurfaceSeries).YData.Clear()
        CType(surfaceChart.Series(0), SurfaceSeries).ZData.Clear()
    End Sub

    Private Sub SliderValueChanged(ByVal sender As Object, ByVal e As RoutedPropertyChangedEventArgs(Of Double))
        If surfaceChart Is Nothing Then
            Return
        End If
        Dim slider As Slider = CType(sender, Slider)
        If slider.Name = "squireSize" Then
            surfaceChart.MeshFaceSize = e.NewValue
        ElseIf slider.Name = "sensitivity" Then
            If surfaceChart.InterpolationType = InterpolationType.None Then
                RandomData(False)
            Else
                surfaceChart.InterpolationSensitivity = e.NewValue
            End If
        ElseIf slider.Name = "groundLevel" Then
            surfaceChart.GroundLevel = e.NewValue
        ElseIf slider.Name = "wireFrameThickness" Then
            surfaceChart.WireFrameThickness = e.NewValue
        End If
    End Sub

    Private Sub pointMergeThreshold_PreviewMouseUp(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
        If surfaceChart.InterpolationType = InterpolationType.None Then
            RandomData(False)
        Else
            surfaceChart.PointMergeThreshold = pointMergeThreshold.Value
        End If
    End Sub

    Private timer As DispatcherTimer
    Private direction As Integer = 1
    Private currentBehavior As Integer = 0

    Private points As List(Of Point3D) = New List(Of Point3D)()
    Private elevations As List(Of Point3D) = New List(Of Point3D)()
End Class
